
// Test for high-speed ethernet communication
// One-byte packet is used to control a PWM output on the AVR. The packet
// is returned to the PC.
// (c) Rogier Schouten, GPL V2


#include <avr/io.h>
#include <avr/interrupt.h>
#ifndef F_CPU
#define F_CPU 12500000
#endif
#include <util/delay.h>
#include "enc28j60.h"

// LED settings
#define LED_PIN PINB1

// Ethernet settings
static uint8_t mymac[6] = {0x52,0x4f,0x47,0x49,0x45,0x52};
static uint8_t from_to_mac[6];
// the buffer for user data
#define BUFFER_SIZE 3

// Ethernet runtime
static uint8_t buf[BUFFER_SIZE];
static uint8_t recvlen;




int main(void)
{
	uint8_t ledon = 0;
	uint8_t j = 0;
	// Set the clock prescaler to divide-by-1
	CLKPR=(1<<CLKPCE);
	CLKPR=0;
	
	_delay_ms(20);
	
	// initialize enc28j60
	enc28j60Init(mymac);
	_delay_ms(50);
		
	// initialize I/O ports
	DDRB |= _BV(LED_PIN);
	PORTB |= _BV(LED_PIN);
	// the whole port C is input
	DDRC=0;
	PORTC=0xff; // pullup on
	// the whole port D is output
	DDRD=0xff;

	// wait until the link is up then send a gratuitous ARP
	// to inform any conected switch about my existence
	while(!enc28j60IsLinkUp());
	_delay_ms(50);
	enc28j60gratuitousARPrequest(mymac);

	//sei();
	while(1)	
	{
		// get the next new packet:
		recvlen = enc28j60PollPacket(BUFFER_SIZE,from_to_mac, buf);
		if (recvlen==0){
			continue;
		}
		// toggle LED to acknowledge reception of packet
		if (ledon)
		{
			PORTB|= (1<<LED_PIN);
			ledon=0;
		}
		else
		{
			PORTB &= ~(1<<LED_PIN);
			ledon=1;
		}
		// set pins on port D according to user data received
		PORTD=buf[1];
		//
		j++;
		//buf[0] contains a sequence number we pass it back
		buf[1]=PINC; // read all 8 pins, PC0 to PC7
		buf[2]=j; // some data for demo purposes
		enc28j60TransmitPacket(BUFFER_SIZE, from_to_mac,buf);
	}
	return (0);
}


